/*

HXED, Version 2.00, a Hexadecimal Editor for Binary Files
Copyright 1991 Anthony F. Stuart, All rights reserved.

Date    Name	Change
910209	AFS	Initial release

*/


/* Define some "universal truths" */

#define TRUE	 1		/* logical true				*/
#define FALSE	 0		/* logical false			*/
#define ERR	 (-1)		/* system error				*/
#define OK	 0		/* system okay				*/
#define EOS	 0		/* end of string			*/

#define NULL	 0		/* amounting to nought			*/
#define EOF	(-1)		/* end of file				*/

/* ASCII Codes for pre-ANSI C Compilers */

#define VT	 0x0b		/* vertical tab				*/
#define FF	 0x0c		/* form feed				*/
#define ESC	 0x1b		/* escape				*/

/* Types of keys to read with get_keystroke */

/* Note: this is currently only used by curses interface to select mode	*/

#define CMD		 0x01	/* accept only a command key		*/
#define DATA		 0x02	/* accept only a data key		*/
#define CMD_OR_DATA	 0x03	/* accept command or data key		*/

typedef unsigned char x_char;	/* type used to represent bytes in file	*/
typedef unsigned long x_addr;	/* type used for address/offset in file	*/

/* Macros */

/* make_byte - convert high and low order nibbles into a byte		*/
/* get_high_nibble - get the high order nibble of a byte		*/
/* get_low_nibble - get the low order nibble of a byte			*/
/* make_printable - convert a character to a printable form		*/
/* get_row_addr - get address of first byte in row for any address	*/
/* min/max - return the maxima or minima of two numbers			*/
/* sub_addr - subtract unsigned addresses without going negative	*/

#define make_byte(h,l)		(((h)<<4) | l)
#define get_high_nibble(c)	(lookup[c >> 4])
#define get_low_nibble(c)	(lookup[c & 0x0f])
#define make_printable(c)	(((c < 0x20) || (c > 0x7e)) ? (def_disp ? def_disp : (c ? c : ' ')) : c)
#define get_row_addr(a)	(((a) / bytes_per_line) * bytes_per_line)
#define min(a,b)		((a) < (b) ? (a) : (b))
#define max(a,b)		((a) > (b) ? (a) : (b))
#define sub_addr(a,b)		((a) < (b) ? 0 : (a) - (b))

#define OUT_BUF_SZ	 82	/* size of display output buffer	*/
#define BUF_SZ		 82	/* size of other arbitrary buffers	*/

#define MAX_NAME_LENGTH	12	/* for footer display purposes		*/
#define BYTES_PER_EXT	3	/* ms-dos filename extension		*/

#define TEMP_NAME_TEMPLATE	"hxedtmp.000"
#define TP		 	 sizeof (TEMP_NAME_TEMPLATE)

/* Definitions used by file access routines */

#define BP_BUF_SZ	 1024	/* size of buffer and temp file slots	*/
#define BP_FILL_SZ	 896	/* fill size, leave room for inserts	*/
#define MOVE_FACTOR	 2	/* fraction of buffer to move in split	*/

#define MIN_BP_SZ	 4	/* minimum number of buffers to allow	*/
#define DEF_BP_SZ	 8	/* default number of buffers		*/

#define FMN struct fmn		/* file map node, see file.h		*/
#define BPN struct bpn		/* buffer pool node, see file.h		*/


