/*

HXED, Version 2.00, a Hexadecimal Editor for Binary Files
Copyright 1991 Anthony F. Stuart, All rights reserved.

Date    Name	Change
910209	AFS	Initial release

*/


#include <sys/types.h>

#include "sd.h"
#include "def.h"
#include "ext.h"
#include "mcb.h"
#include "proto.h"

/* The pattern control block simplifies the argument list for */
/* the search and replace pattern matching routines. */

/* Pattern Control Block */

#define PCB struct pcb

PCB {
	x_char	*pat;			/* addr of pattern string */
	x_char	*pat_end;		/* addr of end of pattern */
	x_char	*rep;			/* replacement text on s&r */
	x_char	*rep_end;		/* end of replacement text */
	x_addr	 txt;			/* addr to start search */
	int	 txt_len;		/* length of matched text */
};

/* The static search and replace buffers are used to retain */
/* search and/or replace text across multiple searches. */

char	 search_buf[BUF_SZ];		/* contents of last search */
char	 replace_buf[BUF_SZ];		/* contents of last replace */

/* The search_bin and replace_bin buffers, declared within the */
/* functions, hold the binary form of the search and replace buffers, */
/* i.e. any escape sequences have been expanded to byte values. */

/* Handle the search command */

get_pattern_and_search ()
{
	PCB	 pcb_buf;
	PCB	*pcb = &pcb_buf;
	x_char	 search_bin[BUF_SZ];

	/* Get a search string from user, return if null */
	
	get_text_input ("Search For: ", search_buf, BUF_SZ);
	if (search_buf[0] == EOS)
		return;

	/* Initialize pattern control block, expand escape sequences */

	pcb->pat = search_bin;
	pcb->pat_end = convert_esc_seq (search_buf, search_bin);
	pcb->txt = this_addr + 1;	/* Don't match text under cursor */
	pcb->txt_len = 0;

	/* Search for text and position to it if found */

	if (search_for_text (pcb))
		set_position (pcb->txt, bytes_per_page / 2);
	else
		display_message ("Pattern not found");
}

/* Handle the replace command */

search_and_replace ()
{
	int	 search_count;
	int	 replace_count;
	int	 replace_this;
	int	 replace_all;

	PCB	 pcb_buf;
	PCB	*pcb = &pcb_buf;
	x_char	 search_bin[BUF_SZ];
	x_char	 replace_bin[BUF_SZ];
	
	/* Get a search string from user, return if null */

	get_text_input ("Search For: ", search_buf, BUF_SZ);
	if (search_buf[0] == EOS)
		return;

	pcb->pat = search_bin;
	pcb->pat_end = convert_esc_seq (search_buf, search_bin);

	/* Get a replace string from user, may be null for null replacement */

	get_text_input ("Replace With: ", replace_buf, BUF_SZ);
	pcb->rep = replace_bin;
	pcb->rep_end = convert_esc_seq (replace_buf, replace_bin);

	/* Initialize pattern control block */

	pcb->txt = this_addr;
	pcb->txt_len = 0;

	search_count = 0;
	replace_count = 0;
	replace_all = FALSE;

	/* While text remaining, search for pattern, position to it, */
	/* prompt for action, replace if desired, etc. */

	while (search_for_text (pcb)) {
		search_count++;
		set_position (pcb->txt, bytes_per_page / 2);
		update_header ();
		replace_this = FALSE;
		if (!replace_all) {
			switch (get_response ("Replace? (no/yes/all/stop) ", "nyas")) {
				case 'n':
					pcb->txt++;
					break;
				case 'y':
					replace_this = TRUE;
					break;
				case 'a':
					replace_all = TRUE;
					break;
				case 's':
					display_message ("Search and replace cancelled");
					return;
			}
		}
		if (replace_this || replace_all) {
			replace_count++;
			replace_text (pcb);
		}
	}

	/* Display a message if no text matching pattern was found */

	if (search_count == 0)
		display_message ("Pattern not found");

	/* Turn on the cursor attribute for the (new) current position */

	display_byte (this_addr, cursor_attr);
}

/* Replace text matching pattern with replace text supplied by user */

replace_text (pcb)
PCB	*pcb;
{
	int	 i;
	x_addr	 rep_len;

	rep_len = pcb->rep_end - pcb->rep;

	/* If original length is less than replace length then overtype */
	/* up to original length and insert from there. */

	if (pcb->txt_len < rep_len) {
		for (i = 0; i < pcb->txt_len; i++) {
			set_byte_value (pcb->txt+i, pcb->rep[i]);
		}
		for (i = pcb->txt_len; i < rep_len; i++) {
			insert_byte_value (pcb->txt+i, pcb->rep[i]);
		}
	}

	/* If original length is exactly equal to replace length then */
	/* overtype each new character over each old character. */

	else if (pcb->txt_len == rep_len) {
 		for (i = 0; i < pcb->txt_len; i++) {
			set_byte_value (pcb->txt+i, pcb->rep[i]);
		}
	}

	/* Otherwise new text is shorter than original text, so */
	/* overtype new text and then delete the extra old chars. */

	else {
		for (i = 0; i < rep_len; i++) {
			set_byte_value (pcb->txt+i, pcb->rep[i]);
		}
		for (i = rep_len; i < pcb->txt_len; i++) {
			delete_byte_addr (pcb->txt + rep_len);
		}
	}

	/* Update text address so that the next search starts after */
	/* the inserted text, possibly avoiding a recursive replace. */

	pcb->txt += rep_len;

	/* Now do the best possible job of displaying the changes, */
	/* some of which may end up split across a page. */

	display_page (base_addr);
}

/* Search for text, the heart of pattern matching */

search_for_text (pcb)
PCB	*pcb;
{
	/* Loop through text and check for pattern at each character */

	while (pcb->txt < mcb->len) {
		if (pattern_match (pcb)) 
			return (TRUE);
		pcb->txt++;
	}

	return (FALSE);
}

/* See if the pattern matches the text starting at pcb->txt */

pattern_match (pcb)
PCB	*pcb;
{
	x_addr	 txt;
	x_char	*pat;

	/* Make local copies of text and pattern to process */

	txt = pcb->txt;
	pat = pcb->pat;

	/* For each char in the pattern */

	while (pat < pcb->pat_end) {

		/* Fail if pattern longer than text */

		if (txt >= mcb->len)
			return (FALSE);

		/* Process special characters in pattern */

		switch (*pat) {
			case '?':
				pat++;
				txt++;
				break;
			case '*':
				return (wildcard_match (pcb, pat, txt));
			case '\\':
				pat++;
				/* fall through */
			default:
				if (*pat != get_byte (txt))
					return (FALSE);
				pat++;
				txt++;
				break;
		}
	}

	/* Return length of matched text */

	pcb->txt_len = txt - pcb->txt;
	return (TRUE);
}

/* Match zero or more characters up to character following * in pattern */

wildcard_match (pcb, pat, txt)
PCB	*pcb;
x_char	*pat;
x_addr	 txt;
{
	PCB	 sub_pcb;

	/* Initialize for a wildcard search based on pattern following "*" */

	sub_pcb.txt = txt;
	sub_pcb.pat = pat + 1;
	sub_pcb.pat_end = pcb->pat_end;

	/* For each character remaining in the search text */

	while (sub_pcb.txt < mcb->len) {

		/* Recursively search for text following wildcard. */

		if (pattern_match (&sub_pcb)) {

			/* Eureka! We've found it, len is sum of sub-match */
			/* length and length of text matched by caller. */

			pcb->txt_len = sub_pcb.txt_len + (sub_pcb.txt - pcb->txt);
			return (TRUE);
		}

		/* Otherwise on to next character */

		sub_pcb.txt++;
	}

	/* Fail if sub pattern not found. */

	return (FALSE);
}


