/*

HXED, Version 2.00, a Hexadecimal Editor for Binary Files
Copyright 1991 Anthony F. Stuart, All rights reserved.

Date    Name	Change
910209	AFS	Initial release

*/


#include <setjmp.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>

#include "sd.h"
#include "def.h"
#include "ext.h"
#include "proto.h"

extern jmp_buf	 error_return;	       /* error return for non-fatal errors */

/* Handle a fatal error from anywhere in the editor */

/* Note: this may leave HXEDtmp files lying around but some of */
/* the errors are severe enough that it is not safe to delete them. */

fatal (message)
char	*message;
{
	static int	 recursive_call = FALSE;
	char		 buf[BUF_SZ];

	/* Check for recursive error condition and exit if it occurs */

	if (recursive_call)
		exit (1);

	recursive_call = TRUE;

	/* Format and display a fatal error message */

	clear_screen (DEFAULT_ATTR);
	display_string (minrow, mincol, "An unrecoverable error occurred:", DEFAULT_ATTR);
	display_string (minrow+2, mincol+5, message, DEFAULT_ATTR);
	display_string (minrow+4, mincol, "Please correct the error condition and edit the file again.", DEFAULT_ATTR);
	position_cursor (minrow+6, mincol);
	restore_hardware ();
	exit (1);
}

/* Handle a recoverable error from anywhere in the editor */

/* Note: this function should only be called from locations */
/* that can return to the main command entry loop. */

error (message)
char	*message;
{
	display_message (message);
	longjmp (error_return, 1);
}

/* Find a temporary file name that is not in use */

/* Note: there is a narrow timing window in networked environments */
/* where it is possible for two HXED's to select the same filename. */

create_temp_name (temp_name)
char	*temp_name;
{
	register int	 j;
	register int	 i;
	int		 n;
	int		 ext_begin;
	int		 ext_end;

	/* Start with a copy of the temporary filename template */

	strcpy (temp_name, TEMP_NAME_TEMPLATE);
	ext_end = strlen (temp_name) - 1;
	ext_begin = ext_end - BYTES_PER_EXT + 1;

	/* Then try up to 1000 times to create a unique temporary filename */

	for (i = 1; i < 1000; i++) {
		for (j = ext_end, n = i; j >= ext_begin; j--) {
			temp_name[j] = '0' + n % 10;
			n /= 10;
		}
		if (access (temp_name, 0) == ERR)
			return;
	}

	/* Hopefully this will never happen, unless there are */
	/* a 1000 people using HXED in a networked environment! */

	fatal ("Cannot create temp name");
}

/* See if a pathname is valid */

validate_path_name (path_name)
char	*path_name;
{
	int	 fd;

	/* It's valid if it already exists */

	if (access (path_name, 0) == OK)
		return;

	/* It's not valid if it cannot be created */

	fd = open (path_name, O_CREAT|O_WRONLY, S_IREAD|S_IWRITE);
	if (fd == ERR)
		error ("Cannot create file");

	/* So if it can be created, close it, and leave no signs */
 
	close (fd);
	unlink (path_name);
}


